// -*- mode:c++;tab-width:2;indent-tabs-mode:t;show-trailing-whitespace:t;rm-trailing-spaces:t -*-
// vi: set ts=2 noet:
//
// (c) Copyright Rosetta Commons Member Institutions.
// (c) This file is part of the Rosetta software suite and is made available under license.
// (c) The Rosetta software is developed by the contributing members of the Rosetta Commons.
// (c) For more information, see http://www.rosettacommons.org. Questions about this can be
// (c) addressed to University of Washington UW TechTransfer, email: license@u.washington.edu.

/// @file EdaAbrelax
/// @brief Application-level code for EdaAbrelax protocol
///
/// @author David Simoncini

#ifndef EdaAbrelaxApplication_hh
#define EdaAbrelaxApplication_hh

// Unit Headers

// Package Headers

// Project Headers
//#include <utility/pointer/boost/owning_ptr.hh>
//#include <utility/pointer/boost/access_ptr.hh>
//#include <utility/pointer/boost/ReferenceCount.hh>

#include <protocols/jobdist/JobDistributors.hh> // keep first

#include <core/pose/Pose.fwd.hh>
#include <core/types.hh>
#include "EdaPose.fwd.hh"
#include "EdaAbinitio.fwd.hh"
#include <protocols/jobdist/Jobs.fwd.hh>

#include <core/io/silent/SilentFileData.fwd.hh>

#include <core/scoring/constraints/ConstraintSet.fwd.hh>
//#include <core/scoring/constraints/ConstraintForest.fwd.hh>
#include <core/scoring/ScoreFunction.fwd.hh>
// AUTO-REMOVED #include <core/scoring/MembraneTopology.fwd.hh>
#include <core/fragment/FragSet.fwd.hh>

#include <protocols/evaluation/PoseEvaluator.fwd.hh>
#include <protocols/evaluation/PCA.fwd.hh>

// AUTO-REMOVED #include <protocols/abinitio/ClassicAbinitio.fwd.hh>
// AUTO-REMOVED #include <protocols/abinitio/MembraneAbinitio.fwd.hh>
#include <protocols/abinitio/Protocol.fwd.hh>
#include <protocols/abinitio/AbrelaxApplication.hh>
#include <protocols/abinitio/Templates.fwd.hh>

/// The instance of Loops contained by AbrelaxApplication should be replaced by a LoopsOP
#include <protocols/loops/Loops.hh>

/// The instance of CheckPointer contained by AbrelaxApplication should be replaced by a CheckPointerOP
#include <protocols/checkpoint/CheckPointer.hh>

#include <protocols/jumping/JumpSetup.fwd.hh>
#include <protocols/jumping/MembraneJump.fwd.hh>
#include <core/fragment/SecondaryStructure.fwd.hh>

// ObjexxFCL Headers

// Utility headers
// #include <utility/pointer/ReferenceCount.hh>

//// C++ headers
// AUTO-REMOVED #include <cstdlib>
#include <string>
#include <vector>

#include <utility/vector1.hh>

namespace protocols{
	namespace abinitio{

class EdaAbrelax: public protocols::abinitio::AbrelaxApplication  {
public:
	typedef protocols::abinitio::AbrelaxApplication Parent;
	EdaAbrelax();
	~EdaAbrelax();
	EdaAbrelax( EdaAbrelax const & src );
	/// @brief setup everything needed for fold() --- calls helper functions below
	void setup_fold( EdaPose &extended_pose, EdaAbinitioOP& prot_ptr );

	void fold( EdaPose &init_pose, EdaAbinitioOP prot_ptr );

	void generate_extended_pose( EdaPose &extended_pose, std::string const& sequence ) const;


	void run();

};
	}
}
#endif
